#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['darkstar']

package_data = \
{'': ['*']}

package_dir = \
{'': 'src'}

install_requires = \
['starlette', 'starlette-htmx', 'jinja2', 'click']

entry_points = \
{'console_scripts': ['darkstar = darkstar.__main__:main']}

setup(name='darkstar',
      version='0.1.5',
      description='Dark Star',
      author=None,
      author_email='Felix Ingram <f.ingram@gmail.com>',
      url=None,
      packages=packages,
      package_data=package_data,
      package_dir=package_dir,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='>=3.6',
     )
